import React, { useEffect, useState } from "react";
import {
  Platform,
  ToastAndroid,
  PermissionsAndroid,
  NativeModules
} from "react-native";

const { Alice: RNOnboarding } = NativeModules;

function haveNonGrantedPermission(permissions) {
  return (
    Object.values(permissions).filter(response => response !== "granted")
      .length > 0
  );
}

function addComponentInfoConfig(config) {
  var pjson = require('./package.json');
  let componentInfoConfig = {
    "componentName": "onboarding-react-native", 
    "componentVersion": pjson.version
  }
  return {...config, ...componentInfoConfig}
}

function parseConfig(config) {
  if (Platform.OS === "android") {
    const { appearance, localization, ...rest } = config;
    return addComponentInfoConfig(rest);
  }
  return addComponentInfoConfig(config);
}

function handleAliceError(error, onCancel, onFailure) {
  if (error.message == "cancel") {
    onCancel(error.message);
  } else {
    onFailure(error.message);
  }
}

function usePermissions() {
  const [granted, setGranted] = useState(false);
  useEffect(() => {
    async function permissionsAsync() {
      // iOS permissions are checked natively
      if (Platform.OS === "android") {
        const response = await PermissionsAndroid.requestMultiple([
          PermissionsAndroid.PERMISSIONS.CAMERA,
          PermissionsAndroid.PERMISSIONS.WRITE_EXTERNAL_STORAGE
        ]);
        if (haveNonGrantedPermission(response)) {
          return ToastAndroid.show(
            "You must accept all permissions to be able to use Alice SDK",
            ToastAndroid.SHORT
          );
        }
      }
      setGranted(true);
    }
    permissionsAsync();
  });
  return granted;
}

export default ({ userToken, config, onSuccess, onFailure, onCancel }) => {
  const granted = usePermissions();
  useEffect(() => {
    async function aliceAsync() {
      try {
        const response = await RNOnboarding.runOnboarding(
          userToken,
          parseConfig(config)
        );
        onSuccess(response);
      } catch (error) {
        handleAliceError(error, onCancel, onFailure);
      }
    }

    if (granted) {
      aliceAsync();
    }
  }, [granted]);

  return null;
};

export function OnboardingWithSandbox({
  sandboxToken,
  email,
  config,
  onSuccess,
  onFailure,
  onCancel,
  firstName = "",
  lastName = ""
}) {
  const granted = usePermissions();
  useEffect(() => {
    async function aliceAsync() {
      try {
        const userToken = await RNOnboarding.logInWithSandbox(
          sandboxToken,
          email,
          firstName,
          lastName
        );
        const response = await RNOnboarding.runOnboarding(
          userToken,
          parseConfig(config)
        );
        onSuccess(response);
      } catch (error) {
        handleAliceError(error, onCancel, onFailure);
      }
    }

    if (granted) {
      aliceAsync();
    }
  }, [granted]);

  return null;
}
